@@ -5,7 +5,7 @@ import tempfile
 import pytest
 
 from .chunker import cf
-from ..chunker import Chunker, ChunkerFixed, sparsemap, has_seek_hole
+from ..chunker import Chunker, ChunkerFixed, sparsemap, has_seek_hole, get_chunker
 from ..constants import *  # NOQA
 
 BS = 4096  # fs block size
@@ -140,7 +140,7 @@ def test_chunkify_sparse(tmpdir, fname, sparse_map, header_size, sparse):
 
 def test_buzhash_chunksize_distribution():
     data = os.urandom(1048576)
-    min_exp, max_exp, mask = 10, 16, 14  # chunk size target 16kiB, clip at 1kiB and 64kiB
+    min_exp, max_exp, mask = 10, 16, 14  # chunk size target 16 KiB, clip at 1 KiB and 64 KiB
     chunker = Chunker(0, min_exp, max_exp, mask, 4095)
     f = BytesIO(data)
     chunks = cf(chunker.chunkify(f))
@@ -153,7 +153,7 @@ def test_buzhash_chunksize_distribution():
     max_count = sum(int(size == 2 ** max_exp) for size in chunk_sizes)
     print(f"count: {chunks_count} min: {min_chunksize_observed} max: {max_chunksize_observed} "
           f"min count: {min_count} max count: {max_count}")
-    # usually there will about 64 chunks
+    # usually there will be about 64 chunks
     assert 32 < chunks_count < 128
     # chunks always must be between min and max (clipping must work):
     assert min_chunksize_observed >= 2 ** min_exp
@@ -161,3 +161,44 @@ def test_buzhash_chunksize_distribution():
     # most chunks should be cut due to buzhash triggering, not due to clipping at min/max size:
     assert min_count < 10
     assert max_count < 10
+
+
+@pytest.mark.skipif("BORG_TESTS_SLOW" not in os.environ, reason="slow tests not enabled, use BORG_TESTS_SLOW=1")
+@pytest.mark.parametrize("worker", range(os.cpu_count() or 1))
+def test_fuzz_chunkify(worker):
+    # Fuzz the chunker with random and all-zero data of misc. sizes and seeds 0 or random int32 values.
+    import random
+
+    def rnd_int32():
+        uint = random.getrandbits(32)
+        return uint if uint < 2**31 else uint - 2**32
+
+    seeds = [0] + [rnd_int32() for _ in range(50)]
+    sizes = [random.randint(1, 4 * 1024 * 1024) for _ in range(50)]
+
+    for seed in seeds:
+        chunker = get_chunker(*CHUNKER_PARAMS, seed=seed)
+        for size in sizes:
+            # Random data
+            data = os.urandom(size)
+            with BytesIO(data) as bio:
+                parts = cf(chunker.chunkify(bio))
+            reconstructed = b"".join(parts)
+            print(seed, size)
+            assert reconstructed == data
+
+            # All-same data
+            data = b"\x42" * size
+            with BytesIO(data) as bio:
+                parts = cf(chunker.chunkify(bio))
+            reconstructed = b"".join(parts)
+            print(seed, size)
+            assert reconstructed == data
+
+            # All-zero data
+            data = b"\x00" * size
+            with BytesIO(data) as bio:
+                parts = cf(chunker.chunkify(bio))
+            # parts has just the integer sizes of each all-zero chunk (since fmap is not used, they are data)
+            print(seed, size)
+            assert sum(parts) == size
